#!/usr/bin/env python
#-*- coding:utf-8 -*


"""
	Manage the database connection
	
	G{importgraph}
"""


#	Pyxmaker
#	Copyright (C) 2013 GALODE A.
#
#	This file is part of Expymaker.
# 
#	Pyxmaker is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
# 
#	Pyxmaker is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with Pyxmaker.  If not, see <http://www.gnu.org/licenses/>


import sqlite3
import sys
import os




#=======================================================================#
#                 		classe gérant la BDD               				#
#=======================================================================#
class BddPyxmaker:
	"""
		Allow to access to the SQLite dataBase of Pyxmaker
		
		G{classtree}
	"""
	
	
	#===================================================#
	#                       Init                        #
	#===================================================#
	def __init__(self, path_base) :
		"""
			Init of BddPyxmaker object
			
			PARAMETERS
			==========
				path_base
				---------
					the path of software execution	
						
			RETURNS
			=======
				None
		"""
		path = path_base
		bdd_path = os.path.join(path, "03-BDD/pyxmaker.sqlite")
		
		self.bdd_pyxmaker = sqlite3.connect(bdd_path)
		self.bdd_pyxmaker.text_factory = str
		self.bdd_pyxmaker.row_factory = sqlite3.Row




	#===================================================#
	# 			Lecture de la table about				#
	#===================================================#
	def f_read_about(self):
		"""
			Allow to read the about window information
			
			PARAMETERS
			==========
				None
						
			RETURNS
			=======
				A dictionary that contains the needed informations
				
		"""
		bdd_cursor = self.bdd_pyxmaker.cursor()
		bdd_cursor.execute("SELECT PARAM, VALUE  \
								FROM  ABOUT")
		dict_about = {}
		for r in bdd_cursor:
			dict_about[r[0]] = r[1]	
		
		bdd_cursor.close()
				
							
		return dict_about
		
		
		
		
	#===================================================#
	# 			Lecture de la table config				#
	#===================================================#
	def f_read_config(self):
		"""
			Allow to read the config information
			
			PARAMETERS
			==========
				None
						
			RETURNS
			=======
				A dictionnary that contains the config information
				
		"""
		bdd_cursor = self.bdd_pyxmaker.cursor()
		bdd_cursor.execute("SELECT PARAM, VALUE \
								FROM  CONFIG")
		config_dict = {}
		for r in bdd_cursor:
			config_dict[r[0]] = r[1]
				
		bdd_cursor.close()
								
		return config_dict
	
	
	
	
	#===================================================#
	#      			MAJ de la table config     			#
	#===================================================#
	def p_upd_config(self, param, new_value):
		"""
			Update config table
			
			PARAMETERS
			==========
				param
				-----
					The parameter to update
				new_value
				---------
					the new value to assign to parameter
					
			RETURNS
			=======
				None
		"""
		bdd_cursor = self.bdd_pyxmaker.cursor()
		bdd_cursor.execute("UPDATE CONFIG \
							SET VALUE = ? \
							WHERE PARAM = ?",(new_value, param))			
		bdd_cursor.close()
		self.bdd_pyxmaker.commit()
		
		
		
		
	#===================================================#
	#			lecture du texte pour les menus			#
	#===================================================#
	def f_read_menu(self, language = "FRANCAIS"):
		"""
			Allow to read the text for the software's menu
			
			PARAMETERS
			==========
				language
				--------
					The language selected by user. "FRANCAIS" by default
						
			RETURNS
			=======
				A list that contains all the menu text
				
		"""
		bdd_cursor = self.bdd_pyxmaker.cursor()
		bdd_cursor.execute("SELECT SCREEN, PARAM, VALUE \
								FROM  MENU \
								WHERE LANG = ?", (language,))
		menu_list = []
		for r in bdd_cursor:
			menu_list.append(r)
		
		bdd_cursor.close()
							
		return menu_list



		
	#===================================================#
	#	 Lecture des differentes langues disponible		#
	#===================================================#
	def f_read_lang(self):
		"""
			Allow to read the different allowed languages
			
			PARAMETERS
			==========
				None
						
			RETURNS
			=======
				A list that contains the different allowed languages
				
		"""
		bdd_cursor = self.bdd_pyxmaker.cursor()
		bdd_cursor.execute("SELECT DISTINCT LANG \
								FROM  MENU")
		lang_list = []
		for r in bdd_cursor:
			lang_list.append(r)
		
		bdd_cursor.close()
							
		return lang_list
		
		
		
		
#=======================================================================#
#                 			Main de la classe                 			#
#=======================================================================#
if __name__ == '__main__':
	try:
		path = ""
		bdd = BddPyxmaker(path)
		config = bdd.f_read_config()
		menu = bdd.f_read_menu()
		about = bdd.f_read_about()
		lang = bdd.f_read_lang()
		print config
		a = raw_input("ALL OK")
	except:
		print "erreur0:", sys.exc_info()
		a = raw_input("\nKO")
	
