#!/usr/bin/env python
#-*- coding:utf-8 -*


"""
	Manage pyxmaker software
	
	G{importgraph}
"""


#	Pyxmaker
#	Copyright (C) 2013 GALODE A.
#
#	This file is part of Expymaker.
# 
#	Pyxmaker is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
# 
#	Pyxmaker is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with Pyxmaker.  If not, see <http://www.gnu.org/licenses/>


import os
import sys
import logging

import path_mkr
import ihm_pyxmaker
import bdd_pyxmaker
import cxf_pyxmaker
import setup_pyxmaker
import save_load_pyxmaker




#=======================================================================#
#						Classe gérant le logiciel						#
#=======================================================================#
class PyxmakerSoftware():
	"""
		Class which manage the Pyxmaker software
		
		G{classtree}
	"""
	
	
	#===================================================#
	#  					   Init							#
	#===================================================#
	def __init__(self):
		self.path_base = self.f_init_path()
		logging.basicConfig(filename=os.path.join(self.path_base,"pyxmaker_log.log"), \
				level = logging.DEBUG, format="%(asctime)s-%(name)s-%(levelname)s-%(message)s")
		logging.info("Pyxmaker initialisation")
		
		self.bdd = self.f_init_bdd()
		self.cxf = self.f_init_cxf()
		self.setup = self.f_init_setup()
		
		self.file_save_load = self.f_init_file_save_load()
		
		dict_config, dict_about, list_menu, list_lang= self.f_bdd_read_base()
				
		self.interface = self.f_init_ihm(list_menu, list_lang, dict_config, dict_about)
		
		self.p_start_software()
		
		
		
		
	#===================================================#
	#      Fonction d'init de la classe BddPyxmaker		#
	#===================================================#	
	def f_init_bdd(self):
		"""
			Allow to create a BddPyxmaker object
			
			PARAMETERS
			==========
				None	
						
			RETURNS
			=======
				A BddPyxmaker object
		"""
		logging.info("bdd_pyxmaker module initialisation")
		
		bdd = bdd_pyxmaker.BddPyxmaker(self.path_base)
		return bdd
		
		
		
		
	#===================================================#
	#  	   Fonction d'init de la classe IhmPyxmaker		#
	#===================================================#	
	def f_init_ihm(self, list_menu, list_lang, dict_config, dict_about):
		"""
			Allow to create a IhmPyxmaker object
			
			PARAMETERS
			==========
				list_menu
				---------
					A list that contains the text of the different menu	
				list_lang
				---------
					A list that contains the different allowed languages
				dict_config
				-----------
					A dictionnary that contains the software configuration
				dict_about
				----------
					A dictionnary that contains the about windows informations
						
			RETURNS
			=======
				An IhmPyxmaker object
		"""
		logging.info("ihm_pyxmaker module initialisation")
		
		ihm = ihm_pyxmaker.IhmPyxmaker(list_menu, list_lang, dict_config, dict_about, self.path_base, \
										self.f_config_save, self.f_config_open, self.f_lang_change, \
										self.p_mk_cxf)
		
		return ihm
		
		
		
		
	#===================================================#
	#   Fonction d'init pour obtenir le chamin d'exec	#
	#===================================================#	
	def f_init_path(self):
		"""
			Allow to know the execution path
			
			PARAMETERS
			==========
				None	
						
			RETURNS
			=======
				The execution path
		"""
		path_base = path_mkr.cxf_get_path()
		return path_base
	
	
	
		
	#===================================================#
	#  	  Fonction d'init de la classe CxfPyxmaker		#
	#===================================================#
	def f_init_cxf(self):
		"""
			Allow to create a CxfPyxmaker object
			
			PARAMETERS
			==========
				None	
						
			RETURNS
			=======
				A CxfPyxmaker object
		"""
		logging.info("cxf_pyxmaker module initialisation")
		
		cxf = cxf_pyxmaker.CxfPyxmaker()
		return cxf
		
		
		
		
	#===================================================#
	#    Fonction d'init de la classe SetupPyxmaker		#
	#===================================================#	
	def f_init_setup(self):
		"""
			Allow to create a SetupPyxmaker object
			
			PARAMETERS
			==========
				None	
						
			RETURNS
			=======
				A SetupPyxmaker object
		"""
		logging.info("setup_pyxmaker module initialisation")
		
		setup = setup_pyxmaker.SetupPyxmaker()
		return setup
	
	
	
	
	#===================================================#
	#  	  Fonction d'init de la classe FileSaveLoad		#
	#===================================================#	
	def f_init_file_save_load(self):
		"""
			Allow to create a FileSaveLoad object
			
			PARAMETERS
			==========
				None	
						
			RETURNS
			=======
				A FileSaveLoad object
		"""
		logging.info("save_load_pyxmaker module initialisation")
		
		file_save_load = save_load_pyxmaker.FileSaveLoad()
		return file_save_load
		
		
		
		
	#===================================================#
	#  	  Fonction de lecture des tables de la BDD		#
	#===================================================#	
	def f_bdd_read_base(self):
		"""
			Allow to read the database tables
			
			PARAMETERS
			==========
				None	
						
			RETURNS
			=======
				Two dictionnary for the configuration and about window
				Two list for the text menu, and the allowed languages
		"""
		logging.info("Database reading, config, language, about, menu")
		
		dict_config = self.bdd.f_read_config()
		list_lang = self.bdd.f_read_lang()
		dict_about = self.bdd.f_read_about()
		list_menu = self.bdd.f_read_menu(dict_config["LANG"])
		
		return dict_config, dict_about, list_menu, list_lang
	
	
	
		
	#===================================================#
	#  		Procedure de MAJ de la table config			#
	#===================================================#
	def p_bdd_upd_config(self, param, new_value):
		"""
			Allow to update the config table
			
			PARAMETERS
			==========
				param
				-----
					the parameter to update
				new_value
				---------
					the new value of the parameter	
						
			RETURNS
			=======
				None
		"""
		logging.info("Databse update")
		
		self.bdd.p_upd_config(param, new_value)
	
	
	
	
	#===================================================#
	#  Fonction permettant la sauvegarde de la config	#
	#===================================================#
	def f_config_save(self, path, dict_param):
		"""
			Allow to save project configuration
			
			PARAMETERS
			==========
				path
				----
					The complete path of save file
				dict_param
				----------
					A dictionnary with all needed parameters	
						
			RETURNS
			=======
				None
		"""
		logging.info("Project configuration saving")
		
		self.file_save_load.p_file_save(path, dict_param)
		
		
		
		
	#===================================================#
	# Fonction permettant de charger le fichier de save	#
	#===================================================#
	def f_config_open(self, path):
		"""
			Allow to create the setup directory in the output repertory
			
			PARAMETERS
			==========
				path
				----
					The complete path of save file
						
			RETURNS
			=======
				a dictionnary with all parameters
		"""
		logging.info("Project configuration loading")
		
		dict_param = self.file_save_load.f_file_load(path)
		return dict_param
	
	
	
	
	#===================================================#
	#  		Fonction gérant le changement de langue		#
	#===================================================#
	def f_lang_change(self, lang):
		"""
			Allow to change the IHM language
			
			PARAMETERS
			==========
				lang
				----
					The selected language	
						
			RETURNS
			=======
				None
		"""
		logging.info("Language modification")
		
		self.bdd.p_upd_config("LANG", lang)

		
		
			
	#===================================================#
	#  Procedure lancant la generation de l'executable	#
	#===================================================#	
	def p_mk_cxf(self, dict_param):
		"""
			Allow to create the windows executable
			
			PARAMETERS
			==========
				dict_param
				----------
					A dictionnary with all needed parameters	
						
			RETURNS
			=======
				None
		"""
		try:
			logging.info("cx_freeze structure creation")
			self.cxf.p_struct_creation(dict_param)
			
			logging.info("cx_freeze script creation")
			self.cxf.p_script_creation(dict_param)
			
			logging.info("executable creation with cx_freeze")
			cxf = self.cxf.f_exe_creation(dict_param)
			
			logging.info("standalone(zip) creation")
			self.cxf.p_zip_standalone(dict_param)
			
			logging.info("setup install call")
			setup = self.p_mk_setup(dict_param)
			
			logging.info("cx_freeze structure delete")
			self.cxf.p_struct_del(dict_param)
			
			if (not setup):
				logging.error("Setup creation KO")
				rtr == 1
			else:
				if cxf == 0:
					logging.info("Standalone and setup creation successfull")
				elif cxf == 1:
					logging.error("executable creation KO")
				elif cxf == 2:
					logging.warning("Potentials problems on executable creation")
					
				rtr = cxf
			
			with open(os.path.join(self.path_base,"pyxmaker_log.log"), "r") as src:
				contenu = src.read()
			src.close()
			
			with open(os.path.join(dict_param["STRUCT_PATH"],dict_param["NAME"] + "_creation.log"), "w") as dst:
				dst.write(contenu)
			dst.close()
			
			logging.shutdown()
			
			os.unlink(os.path.join(self.path_base,"pyxmaker_log.log"))
			
			return rtr
			
		except:
			logging.error("standalone and setup creation KO")
			
			with open(os.path.join(self.path_base,"pyxmaker_log.log"), "r") as src:
				contenu = src.read()
			src.close()
			
			with open(os.path.join(dict_param["STRUCT_PATH"],dict_param["NAME"] + "_creation.log"), "w") as dst:
				dst.write(contenu)
			dst.close()
			
			logging.shutdown()
			
			return 1
		
		
		
		
	#===================================================#
	#  	Procedure lancant la generation de l'install	#
	#===================================================#
	def p_mk_setup(self, dict_param):
		"""
			Allow to create the setup install
			
			PARAMETERS
			==========
				dict_param
				----------
					A dictionnary with all needed parameters	
						
			RETURNS
			=======
				None
		"""
		logging.info("setup structure creation")
		self.setup.p_struct_creation(dict_param)
		
		logging.info("setup script creation")
		self.setup.p_script_creation(dict_param)
		
		logging.info("setup creation with Inno Setup")
		setup = self.setup.f_setup_creation(dict_param)
		
		logging.info("setup moving")
		self.setup.p_setup_move(dict_param)
		
		logging.info("setup structure delete")
		self.setup.p_struct_del(dict_param)
		
		return setup
		
		
		
		
	#===================================================#
	#  			Procedure démarrant le logiciel			#
	#===================================================#	
	def p_start_software(self):
		"""
			Allow to start the software
			
			PARAMETERS
			==========
				None	
						
			RETURNS
			=======
				None
		"""
		self.interface.start_ihm()
	
	
	
		
#=======================================================================#
#                 			Main de la classe                 			#
#=======================================================================#	
if __name__ == "__main__":
	software = PyxmakerSoftware()
	
