#!/usr/bin/env python
#-*- coding:utf-8 -*


"""
	Manage the interface with Inno Setup
	
	G{importgraph}
"""


#	Pyxmaker
#	Copyright (C) 2013 GALODE A.
#
#	This file is part of Expymaker.
# 
#	Pyxmaker is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
# 
#	Pyxmaker is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with Pyxmaker.  If not, see <http://www.gnu.org/licenses/>


import os
import sys
import shutil
import subprocess




#=======================================================================#
#				classe gérant la connexion avec Inno Setup				#
#=======================================================================#
class SetupPyxmaker():
	"""
		Class which interface Pyxmaker and Inno Setup
		
		G{classtree}
	"""
	
	
	#===================================================#
	#                       Init                        #
	#===================================================#
	def __init__(self):
		None
		
		
		
		
	#===================================================#
	#       Creation de la structure de dossiers        #
	#===================================================#	
	def p_struct_creation(self, dict_param):
		"""
			Allow to create the setup directory in the output repertory
			
			PARAMETERS
			==========
				dict_param
				----------
					A dictionnary with all needed parameters	
						
			RETURNS
			=======
				None
		"""
		os.mkdir(dict_param["SETUP_PATH"])
		
	
	
	
	#===================================================#
	#                Creation du script                 #
	#===================================================#
	def p_script_creation(self, dict_param):
		"""
			Allow to create the inno setup script adapted for the software
			
			PARAMETERS
			==========
				dict_param
				----------
					A dictionnary with all needed parameters
						
			RETURNS
			=======
				None
		"""
		path = os.path.join(dict_param["SCRIPTS_PATH"], "script_innos.iss")
		with open(path, 'r') as fichier:
			script = fichier.read()
		fichier.close()
		script = script.replace("$$NAME$$", dict_param["NAME"])
		script = script.replace("$$VERSION$$", dict_param["VERSION"])
		script = script.replace("$$AUTHOR$$", dict_param["AUTHOR"])
		script = script.replace("$$WEBSITE$$", dict_param["WEBSITE"])
		script = script.replace("$$EXEC$$",dict_param["SETUP_EXEC_WITHOUT_PY"])
		script = script.replace("$$UID$$",dict_param["SETUP_UID"])
		script = script.replace("$$LICENCE$$",dict_param["SETUP_LICENCE"])
		script = script.replace("$$BEFORE$$",dict_param["SETUP_BEFORE"])
		script = script.replace("$$AFTER$$",dict_param["SETUP_AFTER"])
		script = script.replace("$$SETUP_MENU$$",dict_param["SETUP_MENU"])
		script = script.replace("$$OUTPUT_DIR$$",dict_param["SETUP_EXE_PATH"])
		script = script.replace("$$SETUP_NAME$$",dict_param["SETUP_NAME"])
		
		path_b = os.path.join(dict_param["PJ_PATH"],"build")
		script = script.replace("$$PATH$$",os.path.join(path_b,os.listdir(path_b)[0]))
		
		if dict_param["SETUP_ICON"] <> "":
			script = script.replace("$$SETUP_ICON$$",dict_param["SETUP_ICON"])
		else:
			script = script.replace("SetupIconFile=$$SETUP_ICON$$","")
			
		if dict_param["SETUP_PASSWD"] <> "":
			script = script.replace("$$SETUP_PASSWD$$",dict_param["SETUP_PASSWD"])
		else:
			script = script.replace("Password=$$SETUP_PASSWD$$","")
		
		path = os.path.join(dict_param["SETUP_PATH"],"script.iss")
		with open(path, "w") as fichier:
			fichier.write(script)
		fichier.close()
		
	
	
	
	#===================================================#
	#              Creation de l'install                #
	#===================================================#	
	def f_setup_creation(self, dict_param):
		"""
			Allow to generate the install setup
			
			PARAMETERS
			==========
				dict_param
				----------
					A dictionnary with all needed parameters
						
			RETURNS
			=======
				None
		"""
		path0 = r"%s"%(dict_param["SETUP_PATH"],)
		
		listing = os.listdir(r"C:\Program Files")
		for i in range(len(listing)):
			if listing[i][0:4].lower() == "inno":
				path1 = r"%s"%(os.path.join("\"C:\Program Files", listing[i]),)
		cmd1 = path1 + "\iscc.exe\" " + path0 + "\\script.iss & exit\n"
		stdout_file = open(os.path.join(dict_param["STRUCT_PATH"],"setup_log.log"), "w+")
		proc = subprocess.Popen(cmd1, shell=True, stdout=stdout_file, stderr=subprocess.STDOUT)
		proc.communicate()
		stdout_file.seek(0)
		stdout = stdout_file.read()
		#cwd indique le chemin ou s'execute la commande
		proc.wait()
		#on attend la fin de l'execution de la commande
		
		with open(dict_param["STRUCT_PATH"] + "setup_log.log", "r") as log:
			contenu = log.read()
			if contenu.find("Successful compile"):
				return True
			else:
				return False
		
		log.close()
		
		
	#===================================================#
	#          Deplacement de l'install genere          #
	#===================================================#
	def p_setup_move(self, dict_param):
		"""
			Allow to move the setup between 2 directories
			
			PARAMETERS
			==========
				dict_param
				----------
					A dictionnary with all needed parameters
						
			RETURNS
			=======
				None
		"""
		shutil.copy(os.path.join(dict_param["SETUP_EXE_PATH"], dict_param["SETUP_NAME"] + ".exe"), \
					os.path.join(dict_param["STRUCT_PATH"],dict_param["SETUP_NAME"] + ".exe"))
	
	
	
	
	#===================================================#
	#         Effacement des dossiers temporaires       #
	#===================================================#
	def p_struct_del(self, dict_param):
		"""
			Allow to delete the temporary directories
			
			PARAMETERS
			==========
				dict_param
				----------
					A dictionnary with all needed parameters
						
			RETURNS
			=======
				None
		"""
		shutil.rmtree(dict_param["SETUP_PATH"], True)



	
#=======================================================================#
#                 			Main de la classe                 			#
#=======================================================================#		
if __name__ == "__main__":
	None

